--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule AESINsel
-- Created   9/23/2006 4:06:36 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity AESINsel is port
   (
     L            : in    std_logic;
     R            : in    std_logic;
     SLD          : out   std_logic_vector(1 to 16);
     SI           : out   std_logic_vector(1 to 16);
     LDDAC        : in    std_logic;
     AESSEL       : in    std_logic_vector(2 downto 0);
     AESSELEN     : in    std_logic;
     AESLvalid    : in    std_logic;
     AESRvalid    : in    std_logic
   );
end AESINsel;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of AESINsel is

-- Component Declarations

-- Signal Declarations
   signal chn : std_logic_vector (1 to 2);
   signal ld  : std_logic_vector (1 to 2);

begin
    chn <= L & R;
    ld(1) <= LDDAC when (AESSELEN = '1') and (AESLvalid = '1') else '0';
    ld(2) <= LDDAC when (AESSELEN = '1') and (AESRvalid = '1') else '0';

    process (AESSEL, chn, ld) is
    begin
       case AESSEL is
          when B"000" =>
             SI  <= chn & B"00" & X"000";
             SLD <= ld  & B"00" & X"000";
          when B"001" =>
             SI  <= B"00" & chn & X"000";
             SLD <= B"00" & ld  & X"000";
          when B"010" =>
             SI  <= X"0" & chn & B"00" & X"00";
             SLD <= X"0" & ld  & B"00" & X"00";
          when B"011" =>
             SI  <= X"0" & B"00" & chn & X"00";
             SLD <= X"0" & B"00" & ld  & X"00";
          when B"100" =>
             SI  <= X"00" & chn & B"00" & X"0";
             SLD <= X"00" & ld  & B"00" & X"0";
          when B"101" =>
             SI  <= X"00" & B"00" & chn & X"0";
             SLD <= X"00" & B"00" & ld  & X"0";
          when B"110" =>
             SI  <= X"000" & chn & B"00" ;
             SLD <= X"000" & ld  & B"00" ;
          when others =>
             SI  <= X"000" & B"00" & chn;
             SLD <= X"000" & B"00" & ld;
       end case;
    end process;
end Structure;
--------------------------------------------------------------------------------
