--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule RX_pio_stat_WB
-- Created   9/25/2006 11:44:00 AM
--------------------------------------------------------------------------------
Library IEEE;
   Use IEEE.Std_Logic_1164.all;
   Use IEEE.Numeric_std.all;

    entity RX_pio_stat_WB is port
      (
      CLK                : in    std_logic;
      
      STB_I              : in    std_logic;
      CYC_I              : in    std_logic;
      ACK_O              : out   std_logic;
      ADR_I              : in    std_logic_vector(4 downto 0);
      DAT_I              : in    std_logic_vector(31 downto 0);
      DAT_O              : out   std_logic_vector(31 downto 0);
      WE_I               : in    std_logic;
      CLK_I              : in    std_logic;
      RST_I              : in    std_logic;
      
      FIFO_EN            : out   std_logic;
      FIFO_I             : in    std_logic_vector(15 downto 0);
      FIFO_EOF           : in    std_logic;
      
      ATRIG              : out   std_logic_vector(31 downto 0);
      BTRIG              : out   std_logic_vector(31 downto 0);
      CTRIG              : out   std_logic_vector(31 downto 0);
      DTRIG              : out   std_logic_vector(31 downto 0);
      AMASK              : out   std_logic_vector(31 downto 0);
      BMASK              : out   std_logic_vector(31 downto 0);
      CMASK              : out   std_logic_vector(31 downto 0);
      DMASK              : out   std_logic_vector(31 downto 0);
      AENDM              : out   std_logic_vector(31 downto 0);
      DENDM              : out   std_logic_vector(31 downto 0);
      BENDM              : out   std_logic_vector(31 downto 0);
      CENDM              : out   std_logic_vector(31 downto 0);
      ALEN               : out   std_logic_vector(15 downto 0);
      BLEN               : out   std_logic_vector(15 downto 0);
      CLEN               : out   std_logic_vector(15 downto 0);
      DLEN               : out   std_logic_vector(15 downto 0);
      
      statArbiter        : in    std_logic_vector(15 downto 0);
      statFSM            : in    std_logic_vector(15 downto 0)
      );
   end RX_pio_stat_WB;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
    architecture Structure of RX_pio_stat_WB is
   
   -- Component Declarations
   
   -- Signal Declarations
      signal atrig_i : std_logic_vector(31 downto 0) := (others => '0');
      signal btrig_i : std_logic_vector(31 downto 0) := (others => '0');
      signal ctrig_i : std_logic_vector(31 downto 0) := (others => '0');
      signal dtrig_i : std_logic_vector(31 downto 0) := (others => '0');
      signal amask_i : std_logic_vector(31 downto 0) := (others => '0');
      signal bmask_i : std_logic_vector(31 downto 0) := (others => '0');
      signal cmask_i : std_logic_vector(31 downto 0) := (others => '0');
      signal dmask_i : std_logic_vector(31 downto 0) := (others => '0');
      signal aendm_i : std_logic_vector(31 downto 0) := (others => '0');
      signal bendm_i : std_logic_vector(31 downto 0) := (others => '0');
      signal cendm_i : std_logic_vector(31 downto 0) := (others => '0');
      signal dendm_i : std_logic_vector(31 downto 0) := (others => '0');
      signal alen_i : std_logic_vector(15 downto 0) := (others => '0');
      signal blen_i : std_logic_vector(15 downto 0) := (others => '0');
      signal clen_i : std_logic_vector(15 downto 0) := (others => '0');
      signal dlen_i : std_logic_vector(15 downto 0) := (others => '0');
   
      signal dat_o_i : std_logic_vector(31 downto 0);
   
      signal cnt_adiscard : unsigned (31 downto 0);
      signal cnt_bdiscard : unsigned (31 downto 0);
      signal cnt_cdiscard : unsigned (31 downto 0);
      signal cnt_ddiscard : unsigned (31 downto 0);
      signal cnt_amissing : unsigned (31 downto 0);
      signal cnt_bmissing : unsigned (31 downto 0);
      signal cnt_cmissing : unsigned (31 downto 0);
      signal cnt_dmissing : unsigned (31 downto 0);
      signal cnt_ftype_err: unsigned (31 downto 0);
      signal cnt_flen_err : unsigned (31 downto 0);
      signal cnt_decod_err: unsigned (31 downto 0);
   
   begin
   -- acknowledge
      ACK_O <= STB_I and CYC_I;
   
   -- Read data
   readdata: 
       process (ADR_I, atrig_i, btrig_i, ctrig_i, dtrig_i,
                      amask_i, bmask_i, cmask_i, dmask_i,
                      aendm_i, bendm_i, cendm_i, dendm_i,
                      alen_i, blen_i, clen_i, dlen_i,
                      cnt_adiscard, cnt_bdiscard, cnt_cdiscard, cnt_ddiscard,
                      cnt_amissing, cnt_bmissing, cnt_cmissing, cnt_dmissing,
                      cnt_ftype_err, cnt_flen_err, cnt_decod_err,
                      statArbiter, statFSM, fifo_i, fifo_eof) is
      begin
         case ADR_I is
            when B"00000" =>
               dat_o_i <= atrig_i;
            when B"00001" =>
               dat_o_i <= btrig_i;
            when B"00010" =>
               dat_o_i <= ctrig_i;
            when B"00011" =>
               dat_o_i <= dtrig_i;
            when B"00100" =>
               dat_o_i <= amask_i;
            when B"00101" =>
               dat_o_i <= bmask_i;
            when B"00110" =>
               dat_o_i <= cmask_i;
            when B"00111" =>
               dat_o_i <= dmask_i;
            when B"01000" =>
               dat_o_i <= aendm_i;
            when B"01001" =>
               dat_o_i <= bendm_i;
            when B"01010" =>
               dat_o_i <= cendm_i;
            when B"01011" =>
               dat_o_i <= dendm_i;
            when B"01100" =>
               dat_o_i <= X"0000" & alen_i;
            when B"01101" =>
               dat_o_i <= X"0000" & blen_i;
            when B"01110" =>
               dat_o_i <= X"0000" & clen_i;
            when B"01111" =>
               dat_o_i <= X"0000" & dlen_i;
            when B"10000" =>
               dat_o_i <= std_logic_vector (cnt_adiscard);
            when B"10001" =>
               dat_o_i <= std_logic_vector (cnt_bdiscard);
            when B"10010" =>
               dat_o_i <= std_logic_vector (cnt_cdiscard);
            when B"10011" =>
               dat_o_i <= std_logic_vector (cnt_ddiscard);
            when B"10100" =>
               dat_o_i <= std_logic_vector (cnt_amissing);
            when B"10101" =>
               dat_o_i <= std_logic_vector (cnt_bmissing);
            when B"10110" =>
               dat_o_i <= std_logic_vector (cnt_cmissing);
            when B"10111" =>
               dat_o_i <= std_logic_vector (cnt_dmissing);
            when B"11000" =>
               dat_o_i <= std_logic_vector (cnt_ftype_err);
            when B"11001" =>
               dat_o_i <= std_logic_vector (cnt_flen_err);
            when B"11010" =>
               dat_o_i <= std_logic_vector (cnt_decod_err);
            when B"11110" =>
               dat_o_i <= statFSM & statArbiter;
            when B"11111" =>
               dat_o_i <= X"000" & B"000" & FIFO_EOF & FIFO_I;
            when others =>
               dat_o_i <= (others => '0');
         end case;
      end process readdata;
      DAT_O <= dat_o_i;
   
   -- Write cycle
   writedata: 
       process (CLK_I, RST_I, ADR_I, DAT_I, WE_I) is
      begin
         if rising_edge (CLK_I) then
            if RST_I = '1' then
               atrig_i <= (others => '0');
               btrig_i <= (others => '0');
               ctrig_i <= (others => '0');
               dtrig_i <= (others => '0');
               amask_i <= (others => '0');
               bmask_i <= (others => '0');
               cmask_i <= (others => '0');
               dmask_i <= (others => '0');
               aendm_i <= (others => '0');
               bendm_i <= (others => '0');
               cendm_i <= (others => '0');
               dendm_i <= (others => '0');
            elsif WE_I = '1' then
               case ADR_I is
                  when B"00000" =>
                     atrig_i <= DAT_I;
                  when B"00001" =>
                     btrig_i <= DAT_I;
                  when B"00010" =>
                     ctrig_i <= DAT_I;
                  when B"00011" =>
                     dtrig_i <= DAT_I;
                  when B"00100" =>
                     amask_i <= DAT_I;
                  when B"00101" =>
                     bmask_i <= DAT_I;
                  when B"00110" =>
                     cmask_i <= DAT_I;
                  when B"00111" =>
                     dmask_i <= DAT_I;
                  when B"01000" =>
                     aendm_i <= DAT_I;
                  when B"01001" =>
                     bendm_i <= DAT_I;
                  when B"01010" =>
                     cendm_i <= DAT_I;
                  when B"01011" =>
                     dendm_i <= DAT_I;
                  when B"01100" =>
                     alen_i <= DAT_I (15 downto 0);
                  when B"01101" =>
                     blen_i <= DAT_I (15 downto 0);
                  when B"01110" =>
                     clen_i <= DAT_I (15 downto 0);
                  when B"01111" =>
                     dlen_i <= DAT_I (15 downto 0);
                  when others =>
                  -- nothing
               end case;
            end if;
         end if;
      end process writedata;
   
   -- FIFO read cycle
      FIFO_EN <= '1' when (STB_I = '1') and (CYC_I = '1') and
                       (WE_I = '0') and (ADR_I = B"11111") else '0';
   
   -- export register signals
      ATRIG <= atrig_i;
      BTRIG <= btrig_i;
      CTRIG <= ctrig_i;
      DTRIG <= dtrig_i;
      AMASK <= amask_i;
      BMASK <= bmask_i;
      CMASK <= cmask_i;
      DMASK <= dmask_i;
      AENDM <= aendm_i;
      DENDM <= bendm_i;
      BENDM <= cendm_i;
      CENDM <= dendm_i;
      ALEN  <= alen_i;
      BLEN  <= blen_i;
      CLEN  <= clen_i;
      DLEN  <= dlen_i;
   
   -- Statistics counter for arbiter and microcode state machine
   stat_arbiter: 
       process (CLK, statArbiter, statFSM) is
      begin
         if rising_edge (CLK) then
            if RST_I = '1' then
               cnt_adiscard <= (others => '0');
               cnt_bdiscard <= (others => '0');
               cnt_cdiscard <= (others => '0');
               cnt_ddiscard <= (others => '0');
               cnt_amissing <= (others => '0');
               cnt_bmissing <= (others => '0');
               cnt_cmissing <= (others => '0');
               cnt_dmissing <= (others => '0');
               cnt_ftype_err<= (others => '0');
               cnt_flen_err <= (others => '0');
               cnt_decod_err<= (others => '0');
            else
               if (statArbiter(1) = '1') then
                  cnt_adiscard <= cnt_adiscard + 1;
               end if;
               if (statArbiter(2) = '1') then
                  cnt_bdiscard <= cnt_bdiscard + 1;
               end if;
               if (statArbiter(3) = '1') then
                  cnt_cdiscard <= cnt_cdiscard + 1;
               end if;
               if (statArbiter(4) = '1') then
                  cnt_ddiscard <= cnt_ddiscard + 1;
               end if;
               if (statArbiter(5) = '1') then
                  cnt_ftype_err <= cnt_ftype_err + 1;
               end if;
               if (statArbiter(6) = '1') then
                  cnt_flen_err <= cnt_flen_err + 1;
               end if;
               if (statFSM(0) = '1') then
                  cnt_decod_err <= cnt_decod_err + 1;
               end if;
               if (statFSM(1) = '1') then
                  cnt_amissing <= cnt_amissing + 1;
               end if;
               if (statFSM(2) = '1') then
                  cnt_bmissing <= cnt_bmissing + 1;
               end if;
               if (statFSM(3) = '1') then
                  cnt_cmissing <= cnt_cmissing + 1;
               end if;
               if (statFSM(4) = '1') then
                  cnt_dmissing <= cnt_dmissing + 1;
               end if;
            end if;
         end if;
      end process stat_arbiter;
   
   end Structure;
--------------------------------------------------------------------------------
