--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule TX_FIFO_ready
-- Created   9/11/2006 5:09:40 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
Use IEEE.Numeric_Std.all;

entity TX_FIFO_ready is port
   (
     reset:           in  std_logic;
     wr_clk:          in  std_logic;
     go:              in  std_logic;
     rd_clk:          in  std_logic;
     ready:           out std_logic;
     ack:             in  std_logic;
     err:             out std_logic
   );                                   
end TX_FIFO_ready;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of TX_FIFO_ready is

-- Signal Declarations
   signal go_sync : std_logic_vector (1 downto 0);
   signal counter_err : std_logic;
   signal counter : unsigned (3 downto 0) := (others => '0');
                                 -- max of 16 frames stacked-up

begin
   -- synchronize go signal
   process (rd_clk, go) is
   begin
      if (go = '1') then
         go_sync <= "10";
      elsif rising_edge (rd_clk) then
         go_sync <= '0' & go_sync(1);
      end if;
   end process;

   -- FIFO ready state machine
   tx_FIFO_ready_FSM: process (reset, rd_clk, go_sync, ack) is
   begin
      if (reset = '1') then
         counter_err <= '0';
         counter <= (others => '0');
      elsif rising_edge (rd_clk) then
         counter_err <= '0';
         if (go_sync(0) = '1') and (ack = '0') then
            if (counter = B"1111") then
               counter_err <= '1';
            else
               counter_err <= '0';
               counter <= counter + 1;
            end if;
         elsif (go_sync(1) = '0') and (ack = '1') then
            if (counter = B"0000") then
               counter_err <= '1';
            else
               counter_err <= '0';
               counter <= counter - 1;
            end if;
         end if;
      end if;
   end process tx_FIFO_ready_FSM;

   ready <= '1' when counter /= B"0000" else '0';
   err <= counter_err;

end Structure;

