--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule TX_Trig
-- Created   9/14/2006 6:56:57 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity TX_Trig is port
   (
     CLK         : in    std_logic;
     GO          : out   std_logic;
     FIRST       : out   std_logic;
     LAST        : out   std_logic;
     FLUSH       : out   std_logic;
     CNT         : in    std_logic_vector(25 downto 0);
     TRIG        : in    std_logic_vector(31 downto 0);
     MASK        : in    std_logic_vector(31 downto 0);
     ENDM        : in    std_logic_vector(31 downto 0)
   );
end TX_Trig;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of TX_Trig is

-- Component Declaration
   component TrigQueue port
      (
        CLK         : in    std_logic;
        ENABLE      : in    std_logic;
        CNT         : in    std_logic_vector(25 downto 0);
        TRIG        : in    std_logic_vector(25 downto 0);
        MASK        : in    std_logic_vector(25 downto 0);
        ENDM        : in    std_logic_vector(25 downto 0);
        GO          : out   std_logic;
        FIRST       : out   std_logic;
        LAST        : out   std_logic;
        FLUSH       : out   std_logic
      );
   end component TrigQueue;

-- Signal Declarations
   signal fsm_go : std_logic;
   signal fsm_first : std_logic;
   signal fsm_last : std_logic;
   signal fsm_flush : std_logic;

begin
   -- state machine go
   go_fsm: TrigQueue port map
   (
      CLK => CLK,
      ENABLE => TRIG(0),
      CNT => CNT,
      TRIG => TRIG(31 downto 6),
      MASK => MASK (31 downto 6),
      ENDM => ENDM (31 downto 6),
      GO => fsm_go,
      FIRST => fsm_first,
      LAST => fsm_last,
      FLUSH => fsm_flush
   );
   GO <= fsm_go;
   FIRST <= fsm_first;
   LAST <= fsm_last;
   FLUSH <= fsm_flush;

end Structure;
--------------------------------------------------------------------------------
