--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule TrigQueue
-- Created   9/14/2006 6:56:57 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity TX_TrigQueue is port
   (
     CLK         : in    std_logic;
     GO          : out   std_logic;
     FIRST       : out   std_logic;
     LAST        : out   std_logic;
     FLUSH       : out   std_logic;
     ENABLE      : in    std_logic;
     CNT         : in    std_logic_vector(25 downto 0);
     TRIG        : in    std_logic_vector(25 downto 0);
     MASK        : in    std_logic_vector(25 downto 0);
     ENDM        : in    std_logic_vector(25 downto 0)
   );
end TX_TrigQueue;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of TX_TrigQueue is

-- Signal Declarations
   signal c    : std_logic_vector(25 downto 0);
   signal t    : std_logic_vector(25 downto 0);
   signal ec   : std_logic_vector(25 downto 0);
   signal et   : std_logic_vector(25 downto 0);
   signal cond : std_logic := '0';
   signal begcond : std_logic := '0';
   signal endcond : std_logic := '0';
   signal en : std_logic;
   signal olden : std_logic_vector (1 downto 0);
   signal newen : std_logic := '0';

begin
   c <= MASK and CNT ;
   t <= MASK and TRIG;
   ec <= ENDM and CNT;
   et <= ENDM and TRIG;

   en <= ENABLE;
   process (CLK, cond, t, c, ec, et, endcond, en) is
   begin
      if en = '0' then
         newen <= '0';
         cond <= '0';
         begcond <= '0';
         endcond <= '0';
      elsif rising_edge (CLK) then
         if (t = c) then
            cond <= '1';
            if (ec = et) then
               endcond <= '1';
               begcond <= '0';
            else
               begcond <= endcond;
               endcond <= '0';
            end if;
            if (endcond = '1') and (en = '1') then
               newen <= '1';  -- only start at beginning of cycle
            end if;
         else
            cond <= '0';
         end if;
      end if;
   end process;

   process (CLK, en, olden) is
   begin
      if rising_edge (CLK) then
         olden <= olden(0) & en;
      end if;
   end process;

   GO <= newen and cond;
   FIRST <= begcond;
   LAST <= endcond;
   FLUSH <= olden(1) and not olden(0);

end Structure;
--------------------------------------------------------------------------------
